#include <Application.h>
#include <Button.h>
#include <Clipboard.h>
#include <Entry.h>
#include <fs_attr.h>
#include <Path.h>
#include <Roster.h>
#include <String.h>
#include <StringView.h>
#include <TextControl.h>
#include <View.h>
#include <Window.h>

#include <iostream>
using namespace std;
#include "PlayLog.h"
#include "Playlist.h"

//message constants
#define DESTINATION_BROWSE 'brws'
#define CONFIG_OK 'cfok'
#define CONFIG_CANCEL 'cfcn'
#define SONG_TIMELIMIT_EXPIRE 'sexp'


void *getplugin(void **data,const char *name, const char *, uint32, plugin_info *, SoundPlayBufferedReader* );
void destroyplugin(void*,void *_loader);
BView* configure(BMessage* configData);
void setconfig(void*,BMessage *config);
void getconfig(void*,BMessage *config);

class ConfigureView : public BView
{
public:
	ConfigureView(BMessage* msg);
	~ConfigureView();
	void AttachedToWindow();
	void MessageReceived(BMessage* msg);
	
private:
	BMessage*	fData;
	BTextControl	*fMaxSongs_TC, *fDestination_TC;
	BFilePanel* fFilePanel;
	BButton* fOkBtn, *fCancelBtn, *fBrowseBtn;
	
};


const char *PLUGINID="nhs-PlayLog";

static plugin_descriptor plugin={
	PLUGIN_DESCRIPTOR_MAGIC,PLUGIN_DESCRIPTOR_VERSION,
	PLUGINID,1, PLUGIN_IS_INTERFACE,

	"PlayLog",
	"SoundPlay plugin for the Eavesdropper framework for XML reporting.\n\nBy Michael Armida.\nBased on the ""EventLogger"" plugin by Marco Nelissen.\n\nhttp://www.bebits.com/app/3400\ninfo@nerdherdsoftware.com",
	NULL,
	configure,
	&getplugin,
	&destroyplugin
};

static interface_plugin_ops plugin_ops={
	PLUGIN_INTERFACE_MAGIC, PLUGIN_INTERFACE_VERSION,
	NULL,
	NULL,
	getconfig, //aren't these backwards?!!  cout'ing shows that the API docs have it reversed; the correct order is used here.
	setconfig
};

static plugin_descriptor *plugs[]={
	&plugin,0
};

plugin_descriptor **get_plugin_list(void)
{
	return plugs;
}

void setconfig(void * data,BMessage *config)
{
	((PLLooper*)data) -> SaveSettings(config);
}

void getconfig(void* data,BMessage *config)
{
	((PLLooper*)data) -> LoadSettings(config);
}

BView* configure(BMessage* configData)
{
	ConfigureView* view = new ConfigureView(configData);
	return view;
}

ConfigureView::ConfigureView(BMessage* configData)
	: BView(BRect(0,0,244,147), "config_background", B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW), 
	fData(configData),
	fFilePanel(new BFilePanel(B_OPEN_PANEL, &BMessenger(this), NULL, B_FILE_NODE | B_SYMLINK_NODE, false, NULL, NULL, false, true))
{	
	SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	fMaxSongs_TC = new BTextControl( BRect(8, 9, 184, 28), "maxsongs_tc", "Maximum number of songs: ", NULL, NULL);
	fDestination_TC = new BTextControl( BRect(8, 66, 238, 85), "destination_tc", NULL, NULL, NULL);
	fMaxSongs_TC->SetDivider(150);
	
	//load up data from the message
	int32 maxsongs;
	entry_ref destref;
	if( configData->FindRef("destination", &destref) != B_OK )
	{
		BEntry entry("/boot/home/PlayLog-output");
		entry.GetRef(&destref);
	}
	if( configData->FindInt32("maxsongs", &maxsongs) != B_OK )
	{
		maxsongs = DEFAULT_MAX_SONGS;
	}
	
	BString tempstring; //itoa(...) wasn't available, and I don't wanna do this the hard way.
	tempstring<<maxsongs;
	fMaxSongs_TC->SetText( tempstring.String() );
	BPath temppath( &destref );
	fDestination_TC->SetText( temppath.Path() );
	
	AddChild(fMaxSongs_TC);
	AddChild(fDestination_TC);
	
	BStringView* destlabel = new BStringView(BRect(10, 48, 151, 61), "destlabel", "Output file destination" );
	AddChild(destlabel);

	fBrowseBtn = new BButton( BRect(161, 40, 184, 55), "nhspl:browsebutton", "...", new BMessage(DESTINATION_BROWSE));
	AddChild(fBrowseBtn);
	
	fOkBtn = new BButton( BRect(197, 97, 238, 119), "nhspl:okbutton", "Ok", new BMessage(CONFIG_OK));
	fOkBtn->MakeDefault(true);
	AddChild(fOkBtn);
	fCancelBtn = new BButton( BRect(134, 98, 174, 118), "nhspl:cancelbutton", "Cancel", new BMessage(CONFIG_CANCEL));
	AddChild(fCancelBtn);	
}

void
ConfigureView::AttachedToWindow()
{	
	BMessenger invokerTarget(this, NULL);
	
	fFilePanel->SetTarget(invokerTarget);
	
	fOkBtn->SetTarget(invokerTarget);
	fCancelBtn->SetTarget(invokerTarget);
	fBrowseBtn->SetTarget(invokerTarget);
}

ConfigureView::~ConfigureView()
{
	delete fFilePanel;
}

void
ConfigureView::MessageReceived(BMessage* msg)
{
	switch(msg->what)
	{
		case CONFIG_CANCEL:
			Window()->PostMessage(B_QUIT_REQUESTED); //I don't have to do anything to SP to close the config window, do I?
			break;
			
		case CONFIG_OK:
		{
			//save values in views back to message pointed to by fData
			fData->MakeEmpty();
			fData->AddInt32( "maxsongs", atoi( fMaxSongs_TC->Text() ) );
			entry_ref ref;
			BEntry tempentry( fDestination_TC->Text() );
			tempentry.GetRef(&ref);
			fData->AddRef( "destination", &ref );
			
			//close the window
			Window()->PostMessage(B_QUIT_REQUESTED);
		}
			break;
			
		case B_REFS_RECEIVED:
		{
			entry_ref ref;
			msg->FindRef("refs", &ref);
			BEntry tempEntry (&ref);
			BPath tempPath( &tempEntry );
			fDestination_TC->SetText( tempPath.Path() );
		}
		
			break;
			
		case DESTINATION_BROWSE:
			fFilePanel->Show();
			break;	
			
		default:
			BView::MessageReceived(msg);	
	}	
}

void *getplugin(void **data,const char *name, const char *, uint32, plugin_info *info, SoundPlayBufferedReader* )
{
	PLLooper *looper=new PLLooper(info->controller);
	looper->Run();
	*data=looper;
	
	BClipboard* clipboard = new BClipboard( "eavesdropper" );
	if( clipboard->Lock() ) 
	{
		clipboard->Clear();
		BMessage* clipMsg = clipboard->Data();
		if( clipMsg != NULL )
		{
			clipMsg->AddMessenger( "messenger", BMessenger(looper) );
			clipboard->Commit();
		}
		clipboard->Unlock();
	}
	delete clipboard;
	
	return &plugin_ops;
}

void destroyplugin(void *,void *data)
{
//	cout<<"Playlog: Destroy plugin called\n";
	BClipboard* clipboard = new BClipboard( "eavesdropper" );
	if( clipboard->Lock() )
	{
		clipboard->Clear();
		clipboard->Commit();
		clipboard->Unlock();
	}
	delete clipboard;
	
	PLLooper* looper=(PLLooper*)data;
	looper->PostMessage(B_QUIT_REQUESTED);
}

bool
check_audio_attrs(BFile* file)
{
	attr_info ainfo;
	if( file->GetAttrInfo("Audio:Artist", &ainfo) == B_OK &&
		file->GetAttrInfo("Audio:Album", &ainfo) == B_OK &&
		file->GetAttrInfo("Audio:Title", &ainfo) == B_OK )
		return true;
	return false;
}

BString
get_string_attr(BFile* file, const char* attr)
{
	BString ret;
	attr_info ainfo;
	if( file->GetAttrInfo(attr, &ainfo) == B_OK )
	{
		if( ainfo.type == B_STRING_TYPE )
		{
			char* buf = new char[ainfo.size+1];
			buf[0] = NULL; //ReadAttr won't do this if the attribute is missing.
			file->ReadAttr(attr, B_STRING_TYPE, 0, (void*) buf, ainfo.size+1);
			ret.SetTo(buf);
			delete [] buf;
		}
		else if ( ainfo.type == B_INT32_TYPE ) 
		{
//			cout<<"Int attribute found\n";
			int32 value;
			file->ReadAttr(attr, B_INT32_TYPE, 0, (void*) &value, 4);
			if( value < 10 )
				//convert to double-digit
				ret<<'0'<<value;
			else
				ret<<value;
		}
	}
	return ret;
}

PLLooper::PLLooper(SoundPlayController *controller)
	: BLooper("PlayLog"), fRunner(NULL)
{
	ctrl=controller;
	
	ctrl->Lock();
	ctrl->AddListener(this);
	for(int i=0;;i++)
	{
		PlaylistPtr playlist=ctrl->PlaylistAt(i);
		if(!playlist->Lock())
			break;
		playlist->AddListener(this);
		playlist->Unlock();
	}
	ctrl->Unlock();
	
	LogCurrentSong();
}

PLLooper::~PLLooper()
{
	delete fRunner;
	while( ! fList.empty() )
	{
		delete fList.back();
		fList.pop_back();
	}
}

void 
PLLooper::LogMessage(BString filename)
{
	while( fList.size() >= fMaxSongs )
	{
		delete fList.back();
		fList.pop_back();
	}

	fList.push_front( new log_item(filename, real_time_clock()) );
}

void 
PLLooper::LogMessage(BString artist, BString album, BString title, BString track)
{
	while( fList.size() >= fMaxSongs )
	{
		delete fList.back();
		fList.pop_back();
	}
	
	fList.push_front( new attrib_log_item( artist, album, title, track, real_time_clock() ) );
}

BHandler*
PLLooper::ResolveSpecifier(BMessage* msg, int32 index, BMessage* specifier, int32 what, const char* property)
{
	BPropertyInfo propInfo(prop_list);
	if( propInfo.FindMatch( msg, index, specifier, what, property) >= 0 )
		return this;
	return BLooper::ResolveSpecifier(msg, index, specifier, what, property);	
}

void PLLooper::MessageReceived(BMessage *mes)
{
	// re-add the playlists to the window in case they got orphaned
	switch(mes->what)
	{
		case B_GET_PROPERTY: 
			{
				int32 specindex;
				BMessage specmsg;
				int32 specwhat;
				const char* specprop;
				
				try{
					if( mes->GetCurrentSpecifier( &specindex, &specmsg, &specwhat, &specprop) == B_OK )
					{
						if( strcmp( specprop, "logitem" ) ==0 )
						{
//							cout<<"Generating list of archived log_items...(" << fList.size() << ")\n";
							BMessage reply(B_REPLY);
							reply.AddInt32("error", B_OK);
							for(list<log_item*>::iterator iter = fList.begin(); 
								iter != fList.end();
								iter++)
							{
//								cout<<"\tArchiving...\n";
								BMessage archive;
								if( (*iter)->Archive(&archive) == B_OK )
									reply.AddMessage("result", &archive);
								
							}
							mes->SendReply(&reply);
						}
						else
							throw UnrecognizedScriptException( mes );
					}
					else
						throw UnrecognizedScriptException( mes );
				}
				catch( UnrecognizedScriptException exc )
				{
					BLooper::MessageReceived( exc.fMsg );
				}
			}
			break;
		case PLAYLIST_CURRENT:
			{
				LogCurrentSong();
			}
			break;
		case SONG_TIMELIMIT_EXPIRE:
			{
//				cout<<"Timelimit expired, logging file...\n";
				ctrl->Lock();
				
				PlaylistPtr playlist = ctrl->PlaylistAt(0);
				playlist->Lock();
				
				entry_ref ref;
				get_ref_for_path( playlist->CurrentPath(), &ref );
				BFile file( &ref, B_READ_ONLY );
				if( ! check_audio_attrs( &file ) )
					LogMessage(playlist->CurrentName());
				else
					LogMessage( get_string_attr(&file, "Audio:Artist"),
								get_string_attr(&file, "Audio:Album"),
								get_string_attr(&file, "Audio:Title"),
								get_string_attr(&file, "Audio:Track") );
					
				
				playlist->Unlock();
				ctrl->Unlock();
				
				delete fRunner;
				fRunner = NULL;
			}
			break;

		default:
			BLooper::MessageReceived(mes);
			break;
	}
}

void
PLLooper::LogCurrentSong()
{
	delete fRunner;
				
	ctrl->Lock();
	PlaylistPtr playlist = ctrl->PlaylistAt(0);
	playlist->Lock();
	file_info finfo;
	if( playlist->GetInfo( playlist->CurrentID(), &finfo) != B_OK )
		return;
	playlist->Unlock();
	ctrl->Unlock();
	
	if( finfo.samplerate == 0 )
		return;
	double length = finfo.framecount / finfo.samplerate; //int division, result in seconds
	bigtime_t expireTime;
	if( length > 240 )
		expireTime = 120000000;
	else
		expireTime = (length / 2) * 1000000;
		
//	cout<<"Expiration time set at "<<expireTime<<" usecs.\n";
	
	fRunner = new BMessageRunner(BMessenger(this), &BMessage(SONG_TIMELIMIT_EXPIRE), expireTime);
}

bool PLLooper::QuitRequested()
{
	ctrl->DisableInterface(PLUGINID);
	return false;
}

void 
PLLooper::ReadData(BMessage* msg)
{
	int32 ct;
	type_code type;
	if( msg->GetInfo("logitem", &type, &ct) == B_OK )
	{
		for( int32 ctr=0; ctr<ct; ctr++)
		{
			BMessage archive;
			if( msg->FindMessage("logitem", ctr, &archive) == B_OK )
			{
				log_item* logitem = (log_item*) instantiate_object(&archive);
				fList.push_front(logitem);
			}
		}
	}
}

void
PLLooper::WriteData(BMessage* msg)
{
	for(list<log_item*>::iterator iter = fList.begin(); 
		iter != fList.end();
		iter++)
	{
		//add archived log items to the big message
		log_item* li = *iter;
		BMessage archive;
		if( li->Archive(&archive) == B_OK )
			msg->AddMessage("logitem", &archive);
	}
}

void PLLooper::SaveSettings(BMessage* msg)
{
	msg->MakeEmpty();
	
	msg->AddRef("destination", &fRef);
	msg->AddInt32("maxsongs", fMaxSongs);
	
	BMessage logdata;
	WriteData(&logdata);
	msg->AddMessage("log", &logdata);
	
//	cout<<"Playlog data written to file...\n";
}

void PLLooper::LoadSettings(BMessage* msg)
{
	if( msg->FindRef("destination", &fRef) != B_OK )
	{
		BEntry entry("/boot/home/PlayLog-output");
		entry.GetRef(&fRef);
	}
	
	if( msg->FindInt32("maxsongs", &fMaxSongs) != B_OK )
	{
		fMaxSongs = DEFAULT_MAX_SONGS;
	}
	
	BMessage logdata;
	msg->FindMessage("log", &logdata);
	ReadData(&logdata);
}


int main()
{
	BApplication app("application/x-vnd.marcone-spmixer");
	
	BMessage msg(ENABLE_INTERFACE);
	msg.AddString("interface",PLUGINID);
	be_roster->Launch("application/x-vnd.marcone-soundplay",&msg);
	return 0;
}

