#ifndef PLAYLOG_H
#define PLAYLOG_H

#include <File.h>
#include <FilePanel.h>
#include <MessageRunner.h>
#include <OS.h>
#include <PropertyInfo.h>
#include <String.h>

#include "LogItem.h"

#include "pluginproto.h"

#include <list>
using namespace std;

#define DEFAULT_MAX_SONGS 30

bool
check_audio_attrs(BFile* file);

BString 
get_string_attr(BFile* file, const char* name);

static property_info prop_list [] = 
{
	{ "logitem", {B_GET_PROPERTY, 0}, {B_DIRECT_SPECIFIER, 0}, "Retrieves a list of archived log_item objects."}, 
	0
};	

class PLLooper: public BLooper
{
	private:
		BFile 				fDataFile;
		list<log_item*> 	fList;
		int32 				fMaxSongs;
		entry_ref 			fRef;
		BFilePanel* 		fFilePanel;
		BMessageRunner* 	fRunner;

	public:
									PLLooper(SoundPlayController *);
		virtual 					~PLLooper();
		virtual void 				MessageReceived(BMessage *mes);
		virtual bool 				QuitRequested();
		
		BHandler* 					ResolveSpecifier(BMessage* msg, int32 index, BMessage* specifier, int32 what, const char* property);

		void						LogCurrentSong();
		//using BStrings here because allocation is a problem otherwise, due to context of calls (returns from get_string_attr)
		void 						LogMessage(BString artist, BString album, BString title, BString track);
		void 						LogMessage(BString filename); 
		
		void 						SaveSettings(BMessage* msg);
		void 						LoadSettings(BMessage* msg);
		
		void 						ReadData(BMessage* msg);
		void 						WriteData(BMessage* msg);

		SoundPlayController *ctrl;
};

class UnrecognizedScriptException
{
public:
						UnrecognizedScriptException( BMessage* msg )
							: fMsg(msg)
						{}
	BMessage*			fMsg;
};


// Perhaps it'd be better to asynchronously write to the disk?
// Since only this looper is held up by disk operations, its not a big deal...


#endif
