#include "App.h"

#include "SoundPlayMod.h"
#include "SystemMod.h"

#include <PropertyInfo.h>

static property_info prop_list[] = {
	{ "data-XML", {B_GET_PROPERTY, 0}, {B_DIRECT_SPECIFIER, 0}, "XML version of collected data from all reporting modules."}, 
	0
}; 

App::App()
	: BApplication(APP_SIG)
{
	SoundPlayMod* sp_mod = new SoundPlayMod();
	SystemMod* sys_mod = new SystemMod();
	AddHandler(sp_mod);
	AddHandler(sys_mod);
	fModList.AddItem( (void*) sp_mod );
	fModList.AddItem( (void*) sys_mod );
}

App::~App()
{
	int32 handlerIndex = 0;
	Lock();
	while( handlerIndex < CountHandlers() )
	{
		BHandler* handler = HandlerAt(handlerIndex);
		if( strcmp(handler->Name(), "SoundPlayModule") == 0 || strcmp(handler->Name(), "SystemModule") == 0 )
			delete handler;
		else
			handlerIndex++;
	}
	Unlock();
}

void
App::MessageReceived(BMessage* msg)
{
	switch(msg->what)
	{
		case B_GET_PROPERTY:
		{
			int32 specIndex, specWhat;
			BMessage specMsg;
			const char* specProperty;
			if( msg->GetCurrentSpecifier( &specIndex, &specMsg, &specWhat, &specProperty ) != B_OK )
			{
				//can't have a GET message without specifiers; sorry.
				BMessage reply;
				reply.AddInt32("error", B_BAD_SCRIPT_SYNTAX );
				reply.AddString("message", "No specifier, no data.  Get a clue!");
			}
			if( strcmp(specProperty, "data-XML") == 0 )
			{
				BString output, header, body;
				for( int32 ctr=0; ctr< fModList.CountItems(); ctr++)
				{
					Module* mod = (Module*) fModList.ItemAt(ctr);
					if( mod->Execute(this) == B_OK )
					{
						header<<mod->HeaderXML();
						body<<mod->BodyXML();
					}
				}

				output<<"<eavesdropper><modules>"<<header<<"</modules>"
					<<body<<"</eavesdropper>";
				
				//data is ready
				BMessage replyMsg(B_REPLY);
				replyMsg.AddInt32("error", B_OK);
				replyMsg.AddString("result", output);
				msg->SendReply(&replyMsg);
			}
			break;
		}
		
		default:
			BApplication::MessageReceived(msg);
	}
}


BHandler*
App::ResolveSpecifier(BMessage* msg, int32 index, BMessage* spec, int32 what, const char* property)
{
	BPropertyInfo prop_info(prop_list);
	if( prop_info.FindMatch(msg, index, spec, what, property) >= 0 )
		return this;
	return BApplication::ResolveSpecifier(msg, index, spec, what, property);
}

status_t
App::GetSupportedSuites(BMessage *message) 
{
	message->AddString("suites", "suite/vnd.NHS.eavesdropper");
	BPropertyInfo prop_info(prop_list); 
	message->AddFlat("messages", &prop_info); 
	return BApplication::GetSupportedSuites(message); 
}


Module::Module(const char* name)
	: BHandler(name), fExecStatus(B_NO_INIT), fLastExecution(0) 
{}

BString
Module::HeaderXML() const
{
	BString ret;
	if( fExecStatus == B_OK )
		ret<<"<module><name>"<<Name()<<"</name><date>"<<fLastExecution<<"</date></module>";
	return ret;
}

const BString&
Module::BodyXML() const
{
	return fData;
}


int main()
{
	App app;
	app.Run();
	return 0;
}
