#include "SystemMod.h"

#include <Messenger.h>
#include <OS.h>
#include <Roster.h>

#define SOUNDPLAY_SIG "application/x-vnd.marcone-soundplay"
#define PE_SIG "application/x-vnd.Hekkel-Pe"
#define BEAIM_SIG "application/x-vnd.FifthAce-BeAIM"
#define BESHARE_SIG "application/x-vnd.Sugoi-BeShare"
#define BEIDE_SIG "application/x-mw-BeIDE"
#define SCREENBLANKER_SIG "application/x-vnd.Be.screenblanker"

SystemMod::SystemMod()
	: Module("SystemModule")
{
}

SystemMod::~SystemMod()
{}

status_t
SystemMod::Execute(App* app)
{
	if( fData.CountChars() > 0 )
		fData.Remove( 0, fData.CountChars() );
	fLastExecution = real_time_clock();
	
	fData<<"<" << Name() << ">";
	
	BRoster roster;

	fData<<"<Applications>";
	
	fData<<"<SoundPlay>";
	if( roster.IsRunning( SOUNDPLAY_SIG ) )
	{
		fData<<"<Status>Running</Status>";
		BMessenger soundplay( SOUNDPLAY_SIG );
		BMessage windowCtReq(B_COUNT_PROPERTIES), windowCtResp(B_NO_REPLY);
		windowCtReq.AddSpecifier("Window");
		soundplay.SendMessage(&windowCtReq, &windowCtResp);
		bool foundLiveEncoder = false;
		if( windowCtResp.what == B_REPLY )
		{
			int32 windowCt;
			windowCtResp.FindInt32("result", &windowCt);
			for(int32 ctr=0; ctr < windowCt && !foundLiveEncoder; ctr++)
			{
				BMessage windowReq(B_GET_PROPERTY), windowResp(B_NO_REPLY);
				windowReq.AddSpecifier("Window", ctr);
				soundplay.SendMessage(&windowReq, &windowResp);
				if( windowResp.what == B_REPLY)
				{
					BMessenger window;
					windowResp.FindMessenger("result", &window);
					if( window.IsValid() )
					{
						BMessage windowTitleReq(B_GET_PROPERTY), windowTitleResp(B_NO_REPLY);
						windowTitleReq.AddSpecifier("Title");
						windowTitleReq.AddSpecifier("Window", ctr);
						soundplay.SendMessage(&windowTitleReq, &windowTitleResp);
						if( windowTitleResp.what == B_REPLY )
						{
							const char* windowTitle;
							windowTitleResp.FindString( "result", &windowTitle );
							if( strcmp(windowTitle, "Live Encoder") == 0)
							{
								foundLiveEncoder = true;
								break;
							}
						}
					}
				}
			}
			if( foundLiveEncoder )
				fData<<"<LiveEncoder>Running</LiveEncoder>";
			else
				fData<<"<LiveEncoder>Not running</LiveEncoder>";
		}
	}
	else
		fData<<"<Status>Not running</status>";
	fData<<"</SoundPlay>";
	
	fData<<"<Pe>";
	if( roster.IsRunning(PE_SIG) )
		fData<<"<Status>Running</Status>";
	else
		fData<<"<Status>Not running</Status>";
	fData<<"</Pe>";
	
	fData<<"<BeAIM>";
	if( roster.IsRunning(BEAIM_SIG) )
	{
		fData<<"<Status>Running</Status>";
		BMessenger beaim( BEAIM_SIG );
		BMessage windowCtReq(B_COUNT_PROPERTIES), windowCtResp(B_NO_REPLY);
		windowCtReq.AddSpecifier("Window");
		beaim.SendMessage(&windowCtReq, &windowCtResp);
		bool foundBuddyList = false;
		if( windowCtResp.what == B_REPLY )
		{
			int32 windowCt;
			windowCtResp.FindInt32("result", &windowCt);
			for(int32 ctr=0; ctr < windowCt && !foundBuddyList; ctr++)
			{
				BMessage windowReq(B_GET_PROPERTY), windowResp(B_NO_REPLY);
				windowReq.AddSpecifier("Window", ctr);
				beaim.SendMessage(&windowReq, &windowResp);
				if( windowResp.what == B_REPLY)
				{
					BMessenger window;
					windowResp.FindMessenger("result", &window);
					if( window.IsValid() )
					{
						BMessage windowTitleReq(B_GET_PROPERTY), windowTitleResp(B_NO_REPLY);
						windowTitleReq.AddSpecifier("Title");
						windowTitleReq.AddSpecifier("Window", ctr);
						beaim.SendMessage(&windowTitleReq, &windowTitleResp);
						if( windowTitleResp.what == B_REPLY )
						{
							const char* windowTitle;
							windowTitleResp.FindString( "result", &windowTitle );
							if( strcmp(windowTitle, "Buddy List") == 0)
							{
								foundBuddyList = true;
								break;
							}
						}
					}
				}
			}
			if( foundBuddyList )
				fData<<"<AIM>Online</AIM>";
			else
				fData<<"<AIM>Offline</AIM>";
		}
	}
	else
		fData<<"<Status>Not running</Status>";
	fData<<"</BeAIM>";
	
	fData<<"<BeShare>";
	if( roster.IsRunning(BESHARE_SIG) )
		fData<<"<Status>Running</Status>";
	else
		fData<<"<Status>Not running</Status>";
	fData<<"</BeShare>";

	fData<<"<BeIDE>";
	if( roster.IsRunning(BEIDE_SIG) )
		fData<<"<Status>Running</Status>";
	else
		fData<<"<Status>Not running</Status>";
	fData<<"</BeIDE>";
	
	fData<<"<ScreenBlanker>";
	if( roster.IsRunning(SCREENBLANKER_SIG) )
		fData<<"<Status>Running</Status>";
	else
		fData<<"<Status>Not running</Status>";
	fData<<"</ScreenBlanker>";
	
	fData<<"</Applications>";
	
	fData<<"<Memory>";
	
	system_info sysinfo;
	get_system_info(&sysinfo);
	fData<<"<Max>"<<sysinfo.max_pages * B_PAGE_SIZE<<"</Max>"
		<<"<Used>"<<sysinfo.used_pages * B_PAGE_SIZE<<"</Used>";
	
	fData<<"</Memory>";
	
	fData<<"<Boottime>"<<sysinfo.boot_time / int64(1000000) <<"</Boottime>"
					<<"<Uptime>"<<system_time() / int64(1000000)<<"</Uptime>";
	
	fData<<"<is_computer_on_fire>"<<(int32) is_computer_on_fire()<<"</is_computer_on_fire>";
	
	fData<<"</" << Name() << ">";
	return fExecStatus = B_OK;
}
