#include "LogItem.h"

#include <Message.h>


log_item::log_item( BMessage* msg )
{
	const char* str;
	msg->FindString("title", &str);
	fTitle = str;
	
	msg->FindInt32("date", &fDate);
}

BArchivable*
log_item::Instantiate( BMessage* msg )
{
	return validate_instantiation(msg, "log_item")  ? new log_item(msg) : NULL;
}

status_t
log_item::Archive(BMessage* msg, bool deep = true) const
{
	msg->AddString("title", fTitle);
	msg->AddInt32("date", fDate);
	return BArchivable::Archive(msg, deep);
}

BString
log_item::XMLize() const
{
	BString xml;
	xml<<"<song><date>"<<fDate<<"</date><fileinfo><filename>"<<fTitle<<"</filename></fileinfo></song>\n";
	return xml;
}

attrib_log_item::attrib_log_item( BMessage* msg )
	 : log_item(msg)
{
	const char* str;
	msg->FindString("album", &str);
	fAlbum = str;
	msg->FindString("artist", &str);
	fArtist = str;
	msg->FindString("track", &str);
	fTrack = str;
}

BArchivable*
attrib_log_item::Instantiate( BMessage* msg )
{
	return validate_instantiation(msg, "attrib_log_item")  ? new attrib_log_item(msg) : NULL;
}

status_t
attrib_log_item::Archive(BMessage* msg, bool deep = true) const
{
	msg->AddString("artist", fArtist);
	msg->AddString("album", fAlbum);
	msg->AddString("track", fTrack);
	return log_item::Archive(msg, deep);
}

BString
attrib_log_item::XMLize() const
{
	BString xml; 
	//not getting base classes representation, because we're reinterpreting 
	//the title field as the track title, instead of filename
	xml<<"<song><date>"<<fDate<<"</date><attribs><title>"<<fTitle<<"</title><artist>"<<fArtist<<"</artist><album>"<<fAlbum<<"</album><track>"<<fTrack<<"</track></attribs></song>\n";
	return xml;
}
